
//
//          IOT DEMO UOF
//
//
//  Programme pour IOT démonstrateur à l'UOF
//
//  Architecturé autour d'un ESP32 Firebeetle relié à un DHT22
//
//  Auteur : Laurent MARCHAL
//
//  Date : Septembre 2022
//
//  Licence : CC - By - Sa
//
//
//  V 1.1 : - insertion d'un contrôlede la tension Vcc (pont 100k/200k avec GPIO34).
//              Si la tension est > seuil => on est en charge
//              Si la tension est < seuil => on est sur batterie 
//                La tension de seuil est définie dans les paramètres
//          - insertion d'un flag autorisant ou non l'émission Wifi si la batterie est en charge
//          - modification du flash de la led. S'il est autorisé : 
//                Allumage pendant toute la durée des mesure/envoi des données (environ 3 à 5 sec.)
//                Idem si Raz manuelle et hors charge
//                Flash très bref sur RAZ manuelle si en charge.
//            
//  V 1.0 : version initiale finale.
//            Durée entre 2 émissions paramètrable
//            Pas d'envoi d'infos sur le port série
//            Allumage de la led à l'init
//
//  V 0.7 : correction d'un bug : seul le premier sujet (de la liste des 3) était envoyé sur le broker.
//
//  V 0.6 : ajout d'un time-out pour la connection MQTT (évite de vider la batterie)
//  
//  V 0.5 :  génération d'un ID aléatoire pour les liaisons MQTT
//
//  V 0.4 :   connection au réseau de la fac et au broker raspi de l'UOF
//
//  V 0.3.1 :   Ajout d'une led sur IO5 (vérification de l'allumage)
//              ajout d'une directive pour afficher ou non les informations dans la console (port UART)
//
//  V 0.3 :   ajout du MQTT
//
//  V 0.2 :
//            initialisation du wifi
//            mise en place d'un time out pour la connection
//  V 0.1 : 
//            mise en place des différents fichiers 
//            Réveil toutes les 20 secondes
//            initialisation du capteur et lecture des valeurs
//            affichage dans la console série


#define debug       false     



//--------------------------------------------------------
//          INCLUSIONS
//--------------------------------------------------------
  #include "certificats.h"
  #include "parametres.h"
  #include "credential.h"

  // nécessite la librairie "DHT sensor library for ESPx" disponible ici => http://librarymanager/All#DHTesp
  #include "DHTesp.h"

  // nécessite la librairie "WiFi.h" disponible ici => http://librarymanager/All#WiFi
  #include <WiFi.h>

  // nécessite la bibliothèque "PubSubClient.h" disponible ici => https://github.com/knolleary/pubsubclient
  #include <PubSubClient.h>

#include <string>
#include <cstring>
  
//--------------------------------------------------------
//          INSTANCIATIONS
//--------------------------------------------------------

DHTesp dhtSensor;
WiFiClient espClient;
PubSubClient client(espClient);
  
//--------------------------------------------------------
//          DEFINITIONS
//--------------------------------------------------------
  #define PIN_COMMANDE_GND    25    // GPIO25
  #define PIN_DHT22           21    // GPIO21
  #define uS_TO_S_FACTOR      1000000

  #define PIN_LED_ONOFF       5     // GPIO05
  
//--------------------------------------------------------
//          CONSTANTES
//--------------------------------------------------------

//--------------------------------------------------------
//          VARIABLES
//--------------------------------------------------------
  int valeur_tension_batterie = 0;
  int valeur_tension_vcc=0;

  unsigned long debut_connection_wifi = 0;
  unsigned long debut_connection_broker = 0;

  char c_temperature[10];
  char c_hygrometrie[10];
  char c_batterie[6];
  char c_vcc[6];


//--------------------------------------------------------
//          S E T U P 
//--------------------------------------------------------
void setup() {
  
  // initialisation du port série
    #if (debug) 
      Serial.begin(115200);
      Serial.println();
      delay(200);
    #endif

  // initialise la led ONOFF
    if (allumage_led == true){
      pinMode (PIN_LED_ONOFF,OUTPUT);
      digitalWrite(PIN_LED_ONOFF,HIGH);
    }

  // autorisation du réveil sur timer interne
    esp_sleep_enable_timer_wakeup(intervalle_entre_deux_emission * uS_TO_S_FACTOR);

  // mesure de la tension Vcc
  valeur_tension_vcc = analogRead(34);

  // Bloque l'éxecution du programme si 
  //      tension Vcc > seui et interdiction de programme en charge
  if (valeur_tension_vcc > tension_separation_charge_batterie and bloquage_programme_en_charge == true) {
      mise_en_veille();
  }


  // génère un ID aléatoire si besoin
  if (client_id_aleatoire==true) {
    randomSeed(analogRead(0));
    genere_id_aleatoire();
  }



  // initialisation de la commande d'alimentation du DHT22
    pinMode (PIN_COMMANDE_GND,OUTPUT);

  // alimentation du DHT22
    digitalWrite(PIN_COMMANDE_GND,HIGH);

  // initialisation du DHT22
    dhtSensor.setup(PIN_DHT22, DHTesp::DHT22);
    #if (debug)
      Serial.println("DHT initiated");
    #endif

  // initialisation du Wifi    WiFi.begin(ssid, password);
    #if (debug)
      Serial.println ("Connection au Wifi : ");
    #endif
    debut_connection_wifi = millis();
    WiFi.begin(wifi_ssid, wifi_password);
        
    while (WiFi.status() != WL_CONNECTED) {
      delay(500);
      #if (debug)
        Serial.print(".");
      #endif
      if ((millis() - debut_connection_wifi) > (timeout_connection_wifi * 1000) ){
        #if(debug)
          Serial.println ("");
          Serial.println ("Connection impossible");
        #endif
        mise_en_veille();
      }
    }
    #if (debug)
      Serial.println("Connecté au réseau Wifi");  
    #endif

  // mesure des valeurs renvoyées par le DHT22
  // Après l'avoir alimenté, il faut laisser au moins une seconde au DHT 22 pour pouvoir envoyer une donnée correct.
  // c'est pourquoi on initialise le Wifi avant
    TempAndHumidity dhtdata;
    if (dhtSensor.getStatus() != 0) {
      #if (debug)
        Serial.println("DHT22 error status: " + String(dhtSensor.getStatusString()));
      #endif
    } else {
      dhtdata = dhtSensor.getTempAndHumidity();
    }

  // mesure de la valeur de la tension batterie (en nb de pas du CAN
    valeur_tension_batterie = analogRead(A0);
  
  // initialisation du MQTT
    #if (debug)
      Serial.print ("initialisation MQTT : ");
    #endif
    client.setServer(broker_adresse, broker_port);
    #if (debug)
      Serial.println ("OK");
    #endif
//---------------------------------------------------------------
    
    if (!client.connected()) {
      debut_connection_broker=millis();
      #if (debug)
        Serial.print ("Connection au Broker MQTT : ");
      #endif
      client.connect(client_id, broker_login, broker_password);
      while (!client.connected()){
        delay(500);
        #if (debug)
          Serial.print (".");
        #endif
         // vérification du time out
          if (millis() - (debut_connection_broker*1000) > timeout_connection_broker) {
            #if(debug)
              Serial.println ("");
              Serial.println ("Connection impossible");
            #endif
            mise_en_veille();
          }
      }
      #if (debug)
        Serial.println ("");
      #endif
    }
    #if (debug)
      Serial.println ("Connecté au serveur MQTT");
    #endif
 
  // envoi des informations au broker
    #if (debug)
      Serial.println ("Envoi des informations au broker MQTT");
      Serial.println ("Température      : " + String(dhtdata.temperature,2) + " °C");
    #endif
    dtostrf(dhtdata.temperature, 5, 2, c_temperature);
    client.publish(topic_mqtt1, c_temperature );
    client.disconnect();
    #if (debug)
      Serial.println ("Status de connection : " +String(client.state()));
    #endif
//---------------------------------------------------------------
    if (!client.connected()) {
      debut_connection_broker=millis();
      #if (debug)
        Serial.print ("Connection au Broker MQTT : ");
      #endif
      client.connect(client_id, broker_login, broker_password);
      while (!client.connected()){
        delay(500);
        #if (debug)
          Serial.print (".");
        #endif
         // vérification du time out
          if (millis() - (debut_connection_broker*1000) > timeout_connection_broker) {
            #if(debug)
              Serial.println ("");
              Serial.println ("Connection impossible");
            #endif
            mise_en_veille();
          }
      }
      #if (debug)
        Serial.println ("");
      #endif
    }
    #if (debug)
      Serial.println ("Connecté au serveur MQTT");
    #endif
     #if (debug)
      Serial.println ("Envoi des informations au broker MQTT");
      Serial.println ("Humidité         : " + String(dhtdata.humidity,1) +" %");
    #endif
    dtostrf(dhtdata.humidity, 4, 1, c_hygrometrie);
    client.publish(topic_mqtt2, c_hygrometrie);
    client.disconnect();
    #if (debug)
      Serial.println ("Status de connection : " +String(client.state()));
    #endif
//---------------------------------------------------------------
// Envoi de la tension batterie au broker (mesurée sur A0 à travers R10 et R11
    if (!client.connected()) {
      debut_connection_broker=millis();
      #if (debug)
        Serial.print ("Connection au Broker MQTT : ");
      #endif
      client.connect(client_id, broker_login, broker_password);
      while (!client.connected()){
        delay(500);
        #if (debug)
          Serial.print (".");
        #endif
         // vérification du time out
          if (millis() - (debut_connection_broker*1000) > timeout_connection_broker) {
            #if(debug)
              Serial.println ("");
              Serial.println ("Connection impossible");
            #endif
            mise_en_veille();
          }
      }
      #if (debug)
        Serial.println ("");
      #endif
    }
    #if (debug)
      Serial.println ("Connecté au serveur MQTT");
    #endif
  
    #if (debug)
      Serial.println ("Envoi des informations au broker MQTT");
      Serial.println ("Tension batterie : " + String( valeur_tension_batterie) + " / 4096");
    #endif
    dtostrf(valeur_tension_batterie, 4, 0, c_batterie);
    client.publish(topic_mqtt3, c_batterie);
    client.disconnect();
    #if (debug)
      Serial.println ("Status de connection : " +String(client.state()));
    #endif


//----------------------------------- FIN MQTT
  // coupure de l'alimentation du DHT22
    digitalWrite(PIN_COMMANDE_GND,LOW);

// ---------------- mise en veille
  // mise en veille programmée
    mise_en_veille();
}

//--------------------------------------------------------
//          L O O P
//--------------------------------------------------------
void loop() {
}


void mise_en_veille() {
  #if (debug)
    Serial.println ("Mise en veille programmée...");
    Serial.flush();
  #endif
  // éteint la led
  digitalWrite(PIN_LED_ONOFF,LOW);

  esp_deep_sleep_start();
   
}

//--------------------------------------------------------
//          FONCTIONS
//--------------------------------------------------------

void genere_id_aleatoire() {
  byte a=0;
  #if(debug)
    Serial.println ("Génère ID aléatoire : ");
    Serial.print ("nb chr dans client_id : ");
    Serial.println (String(strlen(client_id)));
  #endif
  for (int i=0; i<strlen(client_id); i++) {
    a=random(65,90);
    client_id[i] = a;
  }
  #if(debug)
    Serial.print ("Nouvel ID : ");
    Serial.println (client_id);
  #endif
}
